<?php

namespace Drp\UserManagement;

use Carbon\Carbon;
use Drp\UserManagement\Contracts\ManagedUser;
use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Events\UserCreated;
use Drp\UserManagement\Events\UserCreatedWithLoginCode;

/**
 * Class UserCreator
 * @package Drp\UserManagement
 */
class UserCreator
{
    /**
     * @var TokenGenerator
     */
    private $tokenGenerator;

    /**
     * UserCreator constructor.
     * @param TokenGenerator $tokenGenerator
     */
    public function __construct(TokenGenerator $tokenGenerator)
    {
        $this->tokenGenerator = $tokenGenerator;
    }

    /**
     * @return \Drp\UserManagement\Contracts\ManagedUser
     */
    public static function getUserModel()
    {
        return app(config('user_management.model'));
    }

    /**
     * Generates a new first time login token for a user
     * @param ManagedUser $user The user Model
     * @return ManagedUser
     */
    public function reIssueLoginCode(ManagedUser $user)
    {
        $user->addLoginToken(
            $this->tokenGenerator->create(),
            $this->getLoginExpiry()
        );

        event(new UserCreatedWithLoginCode($user));

        return $user;
    }

    /**
     * Create a standard user
     * @param array $data The user data
     * @return User
     */
    public function createStandardUser(array $data = [])
    {
        $data['approved'] = config('user_management.requires_approval') ? 0 : 1;

        $user = $this
            ->getUserModel()
            ->create($data);

        event(new UserCreated($user));

        return $user;
    }

    /**
     * Create a new user with a login code
     * @param array $data The user data
     * @return User
     */
    public function createWithLoginCode(array $data = [])
    {
        $data['approved'] = 1;

        $user = $this
            ->getUserModel()
            ->create(array_merge($data, [
                'first_login_token' => $this->tokenGenerator->create(),
                'login_link_expiry' => $this->getLoginExpiry(),
            ]));

        event(new UserCreatedWithLoginCode($user));

        return $user;
    }

    /**
     * Return the token expiry date
     * @return Carbon
     */
    protected function getLoginExpiry()
    {
        return Carbon::now()->addDays(config('user_management.login_token_expiry_days'));
    }
}
